#! /usr/bin/env python
#own modules
import download
import icons
import simple_css_config
#standard modules
import pygtk
import gobject
import gtk
import gtk.glade
import random
import threading
import time
import urllib
pygtk.require("2.0")

class Input_distro_window:
	def __init__(self,downloader,settings):
		self.settings = settings
		self.downloader = downloader
		self.wTree = gtk.glade.XML( "sfdownloader.glade","input_distro")
		self.callbacks = {"on_button_cancel_clicked":self.__callback_cancel,
					"on_button_download_clicked":self.__callback_download,
					"on_distro_list_event_after":self.__callback_distro_list_clicked}
		self.wTree.signal_autoconnect (self.callbacks)
		self.wTree.get_widget("input_distro").set_icon(gtk.gdk.pixbuf_new_from_file("img/tux.png"))
		self.paths = {}
		self.selected = None
		self.__init_distro_list()
		self.distro_thread = Distro_thread(self.distro_list,self.distro_list_model,self)
		self.distro_thread.start()
		
	def set_paths(self,p):
		self.paths = p
		
	def __init_distro_list(self):
		self.distro_list = self.wTree.get_widget("distro_list")
		
		self.distro_list_model = gtk.TreeStore(gtk.gdk.Pixbuf,gobject.TYPE_STRING)
		self.distro_list.set_model(self.distro_list_model)
		renderer=gtk.CellRendererPixbuf()
		column=gtk.TreeViewColumn("",renderer, pixbuf=0)
		column.set_min_width(16)
		column.set_resizable(True)
		self.distro_list.append_column(column)
		renderer=gtk.CellRendererText()
		column=gtk.TreeViewColumn(_("Name"),renderer, text=1)
		column.set_min_width(150)
		column.set_resizable(True)
		self.distro_list.append_column(column)
		
		self.distro_list.set_enable_tree_lines(True)
		self.distro_list_model.set(self.distro_list_model.append(None),0,None,1,_("Please wait..."))

	#callbacks
	def __callback_download(self,widget):
		if self.selected != None:
			self.downloader.add_job("input_distro",[self.selected],"output_single",[])
		self.distro_thread.join()
		self.wTree.get_widget("input_distro").destroy()

	def __callback_cancel(self,widget):
		self.distro_thread.join()
		self.wTree.get_widget("input_distro").destroy()
		
	def __callback_distro_list_clicked(self,widget,event):
		if event.type == gtk.gdk.BUTTON_PRESS:
			selection = self.distro_list.get_selection()
			(model,iter) = selection.get_selected()
			if iter != None and model.get_path(iter) in self.paths:
				self.selected = self.paths[model.get_path(iter)]
			else:
				self.selected = None

class Input_distro:
	def __init__(self,params,job_id,downloader,settings):
		self.settings = settings
		(self.distro,self.platform,self.title) = params[0]
		self.job_id = job_id
		self.downloader = downloader
		self.url = ""

	def get_downloads(self):
		time.sleep(0.5)
		self.downloader.get_job(self.job_id).set_message(_("Selecting mirror"))
		self.url = self.__get_mirror()
		dl = download.Single_download(self.url,self.job_id,self.title,self.downloader,self.settings)
		self.downloader.get_job(self.job_id).set_message("")
		return [dl]
		
	def __get_mirror(self):
		mirror = ""
		l = self.__get_mirror_list()
		if len(l) == 1:
			return l[0]
		tried = []
		works = False
		i = random.randint(0,len(l)-1)
		while not works and len(tried) != len(l):
			while i in tried:
				i = random.randint(0,len(l)-1)
			f = urllib.urlopen(l[i])
			info = f.info()
			if info.getmaintype() == "application":
				works = True
				break
			tried.append(i)
		mirror = l[i]
		return mirror
		
	def __get_mirror_list(self):
		l = []
		file = open("data/mirrors","r")
		lines = file.readlines()
		record = False
		for x in lines:
			x = x.strip()
			if record and x != "" and x[0] == "[":
				record = False
			if record and x.startswith("http://"):
				l.append(x)
			if x == ("[%s-%s]" % (self.distro,self.platform)):
				record = True
		return l

class Distro_thread(threading.Thread):
	def __init__(self,list,list_model,window):
		threading.Thread.__init__(self)
		self.list = list
		self.window = window
		self.list_model = list_model
		self.__stop_event = threading.Event()

	def run(self):
		cd = icons.pixbuf_from_iconname("media-cdrom")
		
		self.list_model = gtk.TreeStore(gtk.gdk.Pixbuf,gobject.TYPE_STRING)		
		self.paths = {}
		
		self.data = simple_css_config.ConfigFile("data/distros.conf")
		for x in self.data.getSections():
			title = self.data.readString(x,"title")
			platforms = self.data.readString(x,"platforms").split(",")
			icon = gtk.gdk.pixbuf_new_from_file(self.data.readString(x,"img","img/tux.png"))
			iter_distro = self.list_model.append(None)
			self.list_model.set(iter_distro,0,icon,1,title)
			
			for y in platforms:
				iter_plat = self.list_model.append(iter_distro)
				self.list_model.set(iter_plat,0,cd,1,y)
				p = self.list_model.get_path(iter_plat)
				self.paths[p] = (x,y,"%s, %s" % (title,y))
		
		self.list.set_model(self.list_model)
		self.window.set_paths(self.paths)

	def join(self,timeout=None):
		self.__stop_event.set()
		threading.Thread.join(self, timeout)
