#! /usr/bin/env python
#own modules
import download
import icons
import misc
#standard modules
import pygtk
import gtk
import gtk.glade
import time
import urllib
pygtk.require("2.0")

class Input_youtube_window:
	def __init__(self,downloader,settings):
		self.settings = settings
		self.audio_list = None
		self.downloader = downloader
		self.wTree = gtk.glade.XML( "sfdownloader.glade","input_youtube")
		self.callbacks = {"on_button_download_clicked":self.__callback_download,
					  "on_button_cancel_clicked":self.__callback_cancel,
					  "on_select_output_mp3_toggled":self.__callback_output_audio_toggled,
					  "on_select_audio_format_changed":self.__callback_audio_format_selected,
					  "on_select_output_transcode_toggled":self.__callback_transcode_toggled}
		self.wTree.signal_autoconnect (self.callbacks)
		self.clipboard = gtk.Clipboard()
		self.wTree.get_widget("input_youtube").set_icon(icons.pixbuf_from_iconname("video-x-generic"))
		
		self.ffmpeg = misc.app_installed("ffmpeg",self.settings)
		self.mplayer = misc.app_installed("mplayer",self.settings)
		(self.acodecs,self.vcodecs) = misc.get_ffmpeg_encoders()
		self.codec_file_ext = {"flv":".flv","h261":".avi","h263":".avi","h264":"avi","libtheora":".ogg","mjpeg":".avi",
							"msmpeg4":".mpg","msmpeg4v1":".mpg","msmpeg4v2":".mpg",
							"png":".avi","wmv1":".wmv","wmv2":".wmv","xvid":".avi",
							"mpeg1video":".mpg","mpeg2video":".mpg","mpeg4":".mpg"}
		
		self.clipboard.request_text(self.__callback_clipboard)
		if (not self.ffmpeg) and (not self.mplayer):
			self.wTree.get_widget("select_output_mp3").set_sensitive(False)
		if not self.ffmpeg:
			self.wTree.get_widget("select_output_transcode").set_sensitive(False)
		self.__init_audio_list()
		self.__init_video_video_list()

	def __init_audio_list(self):
		liststore = gtk.ListStore(str)
		renderer = gtk.CellRendererText()
		self.wTree.get_widget("select_audio_format").pack_start(renderer,True)
		self.wTree.get_widget("select_audio_format").add_attribute(renderer,'text',0)
		self.wTree.get_widget("select_audio_format").set_model(liststore)
		
		if self.ffmpeg or self.mplayer:
			liststore.set(liststore.append(),0,"MP3")
			if misc.app_installed("oggenc",self.settings):
				liststore.set(liststore.append(),0,"OGG")
			liststore.set(liststore.append(),0,"WAV")
			self.wTree.get_widget("select_audio_format").set_active(0)
		self.audio_list = liststore
		
	def __init_video_video_list(self):
		liststore = gtk.ListStore(str)
		renderer = gtk.CellRendererText()
		self.wTree.get_widget("select_video_video_codec").pack_start(renderer,True)
		self.wTree.get_widget("select_video_video_codec").add_attribute(renderer,'text',0)
		self.wTree.get_widget("select_video_video_codec").set_model(liststore)
		for x in self.vcodecs:
			if x in self.codec_file_ext:
				liststore.set(liststore.append(),0,x)
		self.wTree.get_widget("select_video_video_codec").set_active(0)
		self.vcodec_list = liststore
				
		liststore = gtk.ListStore(str)
		renderer = gtk.CellRendererText()
		self.wTree.get_widget("select_video_audio_codec").pack_start(renderer,True)
		self.wTree.get_widget("select_video_audio_codec").add_attribute(renderer,'text',0)
		self.wTree.get_widget("select_video_audio_codec").set_model(liststore)
		for x in self.acodecs:
			liststore.set(liststore.append(),0,x)
		self.wTree.get_widget("select_video_audio_codec").set_active(0)
		self.acodec_list = liststore
		
		self.wTree.get_widget("entry_video_video_size").set_active(0)
		

	#callbacks
	def __callback_download(self,widget):
		url = self.wTree.get_widget("entry_url").get_text()
		if not url.startswith("http://"):
			url = "http://" + url
		id = url[url.rfind("v=")+2:]
		
		vid = self.settings.get_value("dir_download_video")
		vid =vid[vid.rfind("/")+1:]

		if self.wTree.get_widget("select_output_single").get_active():
			self.downloader.add_job("input_youtube",[id],"output_single",["[str]filename=YouTube_%s.flv" % id])
		elif self.wTree.get_widget("select_output_single_video").get_active():
			self.downloader.add_job("input_youtube",[id],"output_single",["[str]dir="+vid,"[str]filename=YouTube_%s.flv" % id,])
		elif self.wTree.get_widget("select_output_mp3").get_active():
			format = self.__get_audio_format()
			q = int(self.wTree.get_widget("entry_audio_quality").get_value())
			self.downloader.add_job("input_youtube",[id],"output_audio",["[str]dir="+vid,"[str]filename=YouTube_%s.flv" % id,"[str]format=%s" % format,"[str]input-type=video","[int]ogg-quality="+str(q)])
		elif self.wTree.get_widget("select_output_transcode").get_active():
			vcodec = self.__get_vcodec()
			acodec = self.__get_acodec()
			size = self.__get_vsize()
			vbr = int(self.wTree.get_widget("entry_video_video_bitrate").get_value())
			abr = int(self.wTree.get_widget("entry_video_audio_bitrate").get_value())
			ext = self.codec_file_ext[vcodec]
			self.downloader.add_job("input_youtube",[id],"output_video",["[str]dir="+vid,"[str]filename=YouTube_%s.flv" % id,"[str]video-codec=%s" % vcodec,"[str]audio-codec=%s" % acodec,"[int]video-bitrate=%s" % vbr,"[int]audio-bitrate=%s" % abr,"[str]ext=%s" % ext,"[str]size=%s" % size])
		self.wTree.get_widget("input_youtube").destroy()

	def __callback_cancel(self,widget):
		self.wTree.get_widget("input_youtube").destroy()

	def __callback_clipboard(self,clipboard,text,data):
		if text != None and (text.startswith("http://www.youtube.com") or text.startswith("http://youtube.com")):
			self.wTree.get_widget("entry_url").set_text(text)
			
	def __callback_output_audio_toggled(self,widget):
		if self.wTree.get_widget("select_output_mp3").get_active():
			self.wTree.get_widget("expander_audio").set_sensitive(True)
		else:
			self.wTree.get_widget("expander_audio").set_sensitive(False)
			
	def __callback_transcode_toggled(self,widget):
		if self.wTree.get_widget("select_output_transcode").get_active():
			self.wTree.get_widget("expander_video").set_sensitive(True)
		else:
			self.wTree.get_widget("expander_video").set_sensitive(False)
			
	def __callback_audio_format_selected(self,widget):
		format = self.__get_audio_format()
		if format == "ogg":
			self.wTree.get_widget("entry_audio_quality").set_sensitive(True)
		else:
			self.wTree.get_widget("entry_audio_quality").set_sensitive(False)
				
	def __get_audio_format(self):
		format = ""
		if self.audio_list != None:
			i = self.wTree.get_widget("select_audio_format").get_active()
			iter = self.audio_list.get_iter((i))
			format = self.audio_list.get_value(iter,0)
		return format.lower()
		
	def __get_vcodec(self):
		format = ""
		if self.vcodec_list != None:
			i = self.wTree.get_widget("select_video_video_codec").get_active()
			iter = self.vcodec_list.get_iter((i))
			format = self.vcodec_list.get_value(iter,0)
		return format
		
	def __get_acodec(self):
		format = ""
		if self.acodec_list != None:
			i = self.wTree.get_widget("select_video_audio_codec").get_active()
			iter = self.acodec_list.get_iter((i))
			format = self.acodec_list.get_value(iter,0)
		return format
		
	def __get_vsize(self):
		s = ""
		s = self.wTree.get_widget("entry_video_video_size").child.get_text()
		return s
			
class Input_youtube(urllib.FancyURLopener):
	def __init__(self,params,job_id,downloader,settings,timeout=3):
		self.settings = settings
		urllib.FancyURLopener.__init__(self)
		self.id = params[0]
		self.job_id = job_id
		self.downloader = downloader
		self.timeout = timeout
		self.real_url = None
		
	def http_error_301(self, url, fp, errcode, errmsg, headers, data=None):
		t = extract_t_parameter(headers["Location"])
		self.set_real_url(t)
	
	def http_error_302(self, url, fp, errcode, errmsg, headers, data=None):
		t = extract_t_parameter(headers["Location"])
		self.set_real_url(t)
		
	def http_error_303(self, url, fp, errcode, errmsg, headers, data=None):
		t = extract_t_parameter(headers["Location"])
		self.set_real_url(t)
		
	def set_real_url(self,t):
		self.real_url = "http://www.youtube.com/get_video?video_id=%s&t=%s" % (self.id,t)

	def get_downloads(self):
		time.sleep(0.5)
		self.downloader.get_job(self.job_id).set_message(_("Connecting to YouTube"))
		self.open("http://youtube.com/v/%s" % self.id)
		time.sleep(self.timeout)
		self.close()
		if self.real_url != None:
			dl = download.Single_download(self.real_url,self.job_id,"http://youtube.com/watch?v=%s" % self.id,self.downloader,self.settings)
			self.downloader.get_job(self.job_id).set_message("")
			return [dl]
		self.downloader.get_job(self.job_id).set_message(_("Error"))
		return []
		
		
def extract_t_parameter(url):
	t = ""
	t = url[url.find("&t=")+3:]
	return t
