#own modules
import misc
#python modules
import os
import time
import shutil

class Output_audio:
	def __init__(self,params,job_id,downloader,filenames,settings):
		self.settings = settings
		self.params = params
		self.downloader = downloader
		self.job_id = job_id
		self.num = 1
		self.cfiles = []
		self.meta = []
		
		self.options = {"dir":self.settings.get_value("dir_download_main"),
					"enumerate":False,
					"enumerate_replace":False,
					"zerofill":2,
					"filename":"",
					"bitrate":"192",
					"input-type":"video",
					"format":"mp3",
					"ogg-quality":7,
					"sampling-rate":44100}

		self.options = misc.param_list_to_dict(params,self.options)

		
		if self.options["dir"][0] != "/":
			self.options["dir"] = self.settings.get_value("dir_download_main") + "/" + self.options["dir"]
		
		filenames.sort()
		if not os.path.exists(self.options["dir"]):
			os.makedirs(self.options["dir"])

		self.downloader.get_job(self.job_id).set_message(_("Moving files"))
		for x in filenames:
			self.__move(x[0],x[1])
		self.downloader.get_job(self.job_id).set_message("")

		if self.options["format"] == "mp3":
			for x in self.cfiles:
				self.extract_mp3(x)
		elif self.options["format"] == "wav":
			for x in self.cfiles:
				self.extract_wav(x)
		elif self.options["format"] == "ogg":
			for x in self.cfiles:
				self.extract_ogg(x)
				
		self.downloader.meta_files_add(self.meta)

	def __move(self,url,temp_filename):
		dir = self.options["dir"]
		if self.options["filename"] == "":
			self.options["filename"] = url[url.rfind("/")+1:]
		target_filename = dir + "/" + self.options["filename"]
		if self.options["enumerate_replace"]:
			(base,ext) = os.path.splitext(target_filename)
			target_filename = dir + "/" + misc.zerofill(self.num,self.options["zerofill"]) + ext
			self.num = self.num + 1
		elif self.options["enumerate"]:
			(base,ext) = os.path.splitext(target_filename)
			target_filename = dir + "/" + base + misc.zerofill(self.num,self.options["zerofill"]) + ext
			self.num = self.num + 1
		try:
			self.downloader.history_add(url,target_filename)
			self.cfiles.append(target_filename)
			shutil.move(temp_filename,target_filename)
			self.meta.append({"path":target_filename,"url":url,"size":os.path.getsize(target_filename),"date":time.time()})
		except:
			pass
			
			
	def extract_wav(self,x):
		self.downloader.get_job(self.job_id).set_message(_("Extracting wav audio"))
		fn = x
		(base,ext) = os.path.splitext(fn)

		if ext == ".flv":
			#flv input - youtube etc.
			ffmpeg_opts = ["ffmpeg","-i",fn,"%s.wav" % base]
			mplayer_opts = ["mplayer","-vo","null","-ao","pcm:file=%s.wav" % base,fn]
			
		if misc.app_installed("ffmpeg"):
			if os.path.exists("%s.wav" % base):
				os.unlink("%s.wav" % base)
			res = os.spawnvp(os.P_WAIT,"ffmpeg",ffmpeg_opts)
		elif misc.app_installed("mplayer"):
			if os.path.exists("%s.wav" % base):
				os.unlink("%s.wav" % base)
			res = os.spawnvp(os.P_WAIT,"mplayer",mplayer_opts)
		if res == 0:
			self.meta.append({"path":"%s.wav" % base,"url":"","size":os.path.getsize("%s.wav" % base),"date":time.time()})
			self.downloader.get_job(self.job_id).set_message("")
		else:
			self.downloader.get_job(self.job_id).set_message(_("Error extracting wav audio"))
				
	def extract_mp3(self,x):
		self.downloader.get_job(self.job_id).set_message(_("Extracting mp3 audio"))
		fn = x
		(base,ext) = os.path.splitext(fn)

		if ext == ".flv":
			#flv input - youtube etc.
			ffmpeg_opts = ["ffmpeg","-i",fn,"-ab","192","-ar","44100","%s.mp3" % base]
			mplayer_opts = ["mplayer","-dumpaudio",fn,"-dumpfile","%s.mp3" % base]
		
		if misc.app_installed("ffmpeg"):
			if os.path.exists("%s.mp3" % base):
				os.unlink("%s.mp3" % base)
			res = os.spawnvp(os.P_WAIT,"ffmpeg",ffmpeg_opts)
		elif misc.app_installed("mplayer"):
			if os.path.exists("%s.mp3" % base):
				os.unlink("%s.mp3" % base)
			res = os.spawnvp(os.P_WAIT,"mplayer",mplayer_opts)
		if res == 0:
			self.meta.append({"path":"%s.mp3" % base,"url":"","size":os.path.getsize("%s.mp3" % base),"date":time.time()})
			self.downloader.get_job(self.job_id).set_message("")
		else:
			self.downloader.get_job(self.job_id).set_message(_("Error extracting mp3 audio"))
			
			
	def extract_ogg(self,x):
		self.extract_wav(x)
		self.downloader.get_job(self.job_id).set_message(_("Converting to ogg audio"))
		fn = x
		(base,ext) = os.path.splitext(fn)
			
		res = -1
		if misc.app_installed("oggenc"):
			if os.path.exists("%s.ogg" % base):
				os.unlink("%s.ogg" % base)
			res = os.spawnvp(os.P_WAIT,"oggenc",["oggenc","%s.wav" % base,"-q",str(self.options["ogg-quality"]),"-o","%s.ogg" % base])
		if res == 0:
			self.meta.append({"path":"%s.ogg" % base,"url":"","size":os.path.getsize("%s.ogg" % base),"date":time.time()})
			self.downloader.get_job(self.job_id).set_message("")
		else:
			self.downloader.get_job(self.job_id).set_message(_("Error converting to ogg audio"))
		os.unlink("%s.wav" % base)
