#! /usr/bin/env python
#own modules
import download
import icons
import input_distro,input_gallery,input_single,input_youtube #input modules
import output_single,output_gallery,output_audio,output_video #output modules
import misc
import simple_css_config
import window_log
import window_settings
import window_statistics
#python modules
import __builtin__
import datetime
import logging
import os
import pygtk
pygtk.require("2.0")
import gobject
import pango
import gtk
import gtk.glade
import hashlib
import sys
import threading
import time
import locale
import gettext

class Downloader:
	def __init__(self):
		#chdir to correct directory
		if sys.argv[0].find("/") != -1:
			wd = sys.argv[0][:sys.argv[0].rfind("/")]
			os.chdir(wd)
			
		#initialize basics
		self.log_list = []
		self.job_list = []
		self.downloads = []
		self.active_dl_count = 0
		self.app_name = "SFDownloader"
		self.app_short_name = "sfdownloader"
		self.icon = "img/logo.svg"
		self.showing = True
		self.__visible_toggled = True
		self.__init_lang()
		self.status_str = {0:_("Paused/Waiting"),1:_("Connecting"),2:_("Downloading"),3:_("Done"),4:_("Connected"),-1:_("Error")}
		self.percentage_all = 0.0
		self.global_settings = misc.Global_settings()
		self.downloaded_bytes_session = 0
		self.history = []

		#initialize configuration file and saved jobs/downloads
		self.__check_first_start()
		self.working_dir = os.path.expanduser("~/.%s" % self.app_short_name)
		self.saved_jobs = simple_css_config.ConfigFile("%s/jobs.conf" % self.working_dir)
		self.saved_downloads = simple_css_config.ConfigFile("%s/downloads.conf" % self.working_dir)
		self.config = simple_css_config.ConfigFile("%s/app.conf" % self.working_dir)
		self.__init_metadata()
		self.__init_history()

		#load settings from configuration file and create directories if necessary
		self.download_dir = self.config.readString("app","download_dir",os.path.expanduser("~/Downloads"))
		self.temp_dir = self.config.readString("app","temp_dir",os.path.expanduser("~/Downloads/.temp"))
		self.gallery_dir = self.config.readString("input-gallery","gallery_dir",os.path.expanduser("~/Downloads/Galleries"))
		self.video_dir = self.config.readString("input-video","video_dir",os.path.expanduser("~/Downloads/Video"))
		self.audio_dir = self.config.readString("input-audio","audio_dir",os.path.expanduser("~/Downloads/Audio"))
		self.max_dl_num = self.config.readInt("app","max_dl_count",4)
		self.clean_on_start_up = self.config.readBool("app","clean_on_startup",False)
		self.max_thread_num = self.config.readInt("app","max_thread_count",10)
		self.website = self.config.readString("app","website","http://downloader.sven-festersen.de")
		self.version = self.config.readString("app","version","0.3")
		self.downloaded_bytes = self.config.readInt("stats","downloaded_bytes",0)
		self.__check_dl_dirs()
		
		#configure logging
		logging.basicConfig(level=logging.DEBUG,format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',datefmt='%m-%d %H:%M',filename=self.working_dir + "/sfdownloader.log",filemode='w')
		console = logging.StreamHandler()
		console.setLevel(logging.INFO)
		formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
		console.setFormatter(formatter)
		logging.getLogger('').addHandler(console)
		
		#initialize global settings
		self.global_settings.set_value("logger_main",logging.getLogger(self.app_name))
		self.global_settings.set_value("logger_input",logging.getLogger(self.app_name + ".input"))
		self.global_settings.set_value("logger_output",logging.getLogger(self.app_name + ".output"))
		self.global_settings.set_value("logger_threading",logging.getLogger(self.app_name + ".threading"))
		self.global_settings.set_value("logger_download",logging.getLogger(self.app_name + ".download"))
		self.global_settings.set_value("dir_download_main",self.download_dir)
		self.global_settings.set_value("dir_download_temp",self.temp_dir)
		self.global_settings.set_value("dir_download_video",self.video_dir)
		self.global_settings.set_value("dir_download_audio",self.audio_dir)
		self.global_settings.set_value("dir_download_gallery",self.gallery_dir)
		self.global_settings.set_value("download_status",{})
		self.global_settings.set_value("stat_dl_count",0)
		self.global_settings.set_value("stat_dl_active_count",0)
		self.global_settings.set_value("stat_dl_speed",0)
		self.global_settings.set_value("stat_downloaded_bytes",0)
		self.global_settings.set_value("stat_downloaded_bytes_session",0)
		self.global_settings.set_value("stat_speed",0)
		
		self.id_tree = misc.Id_tree("root")

		#input and output plugin list
		self.inputs = {"input_single":input_single.Input_single,"input_gallery":input_gallery.Input_gallery,"input_youtube":input_youtube.Input_youtube,"input_distro":input_distro.Input_distro}
		self.outputs = {"output_single":output_single.Output_single,"output_gallery":output_gallery.Output_gallery,"output_audio":output_audio.Output_audio,"output_video":output_video.Output_video}
		
		#initialize glade and gtk
		self.wTree = gtk.glade.XML( "sfdownloader.glade","window_main")
		self.callbacks = {"on_window_main_destroy":self.__callback_quit,
					  "on_button_new_dl_clicked":self.__callback_new_dl_single,
					  "on_dl_list_event_after":self.__callback_dl_list_click,
					  "on_button_pause_dl_clicked":self.__callback_button_pause_clicked,
					  "on_button_start_dl_clicked":self.__callback_button_start_clicked,
					  "on_button_del_dl_clicked":self.__callback_button_del_clicked,
					  "on_button_clear_clicked":self.__callback_button_clear_clicked,
					  "on_button_settings_clicked":self.__callback_button_settings_clicked,
					  "on_filemenu_quit_activate":self.__callback_quit,
					  "on_filemenu_simple_download_activate":self.__callback_new_dl_single,
					  "on_filemenu_gallery_download_activate":self.__callback_new_dl_gallery,
					  "on_filemenu_youtube_video_download_activate":self.__callback_new_dl_youtube,
					  "on_editmenu_remove_dl_activate":self.__callback_button_del_clicked,
					  "on_editmenu_start_dl_activate":self.__callback_button_start_clicked,
					  "on_editmenu_pause_dl_activate":self.__callback_button_pause_clicked,
					  "on_editmenu_clear_dl_activate":self.__callback_button_clear_clicked,
					  "on_editmenu_settings_activate":self.__callback_button_settings_clicked,
					  "on_helpmenu_log_activate":self.__callback_button_log_clicked,
					  "on_helpmenu_info_activate":self.__callback_menu_about_clicked,
					  "on_button_dl_dir_clicked":self.__callback_button_dl_dir_clicked,
					  "on_button_statistics_clicked":self.__callback_button_statistics_clicked,
					  "on_viewmenu_statistics_activate":self.__callback_button_statistics_clicked,
					  "on_viewmenu_sidebar_activate":self.__callback_sidebar_toggled,
					  "on_history_list_event_after":self.__callback_history_list_clicked,
					  "on_button_history_clear_clicked":self.__callback_button_history_clear_clicked}
		self.wTree.signal_autoconnect (self.callbacks)
		

		self.wTree.get_widget("window_main").set_title(self.app_name)
		self.wTree.get_widget("window_main").set_icon(gtk.gdk.pixbuf_new_from_file(self.icon))
		if self.config.readBool("app","sidebar-show",False):
			pos = self.config.readInt("app","sidebar-position",400)
			self.wTree.get_widget("sidebar").set_position(pos)
			self.wTree.get_widget("viewmenu_sidebar").set_active(True)
		
		self.__init_statusbar()
		self.__create_tray()
		self.__create_dl_list()
		self.__create_dl_menu()
		self.__init_history_list()
		self.wTree.get_widget("editmenu_remove_dl").set_sensitive(False)
		self.wTree.get_widget("editmenu_start_dl").set_sensitive(False)
		self.wTree.get_widget("editmenu_pause_dl").set_sensitive(False)

		self.updater = Update_thread(self,self.global_settings)
		self.updater.start()
		
		self.global_settings.get_value("logger_main").info(_("%s started!") % self.app_name)
		self.global_settings.get_value("logger_main").info(_("Hello."))
		self.load_saved_jobs()
		gtk.gdk.threads_init()
		gtk.main()
		
	def __init_statusbar(self):
		"""This initializes the statusbar of the main window"""
		bar = self.wTree.get_widget("statusbar")
		id = bar.get_context_id("info")
		self.global_settings.set_value("statusbar_id",id)
		
	def __init_metadata(self):
		"""Initialize a global metadata storage"""
		self.meta = simple_css_config.ConfigFile("%s/metadata.conf" % self.working_dir)
		for x in self.meta.getSections():
			if not os.path.exists(x):
				self.meta.removeSection(x)
				
	def __init_history(self):
		if os.path.exists(self.working_dir + "/history"):
			f = open(self.working_dir + "/history","r")
			lines = f.readlines()
			for line in lines:
				line = line.strip()
				if line != "":
					a = line.split("\t")
					self.history.append((a[0],a[1],a[2],a[3]))
			f.close()
		
	def __init_history_list(self):
		list = gtk.TreeStore(gtk.gdk.Pixbuf,str,str)
		hist = self.wTree.get_widget("history_list")
		renderer=gtk.CellRendererPixbuf()
		column=gtk.TreeViewColumn("",renderer, pixbuf=0)
		column.set_min_width(16)
		column.set_resizable(True)
		hist.append_column(column)
		renderer=gtk.CellRendererText()
		renderer.ellipsize = pango.ELLIPSIZE_END
		renderer.ellipsize_set = True
		column=gtk.TreeViewColumn(_("URL"),renderer, text=1)
		column.set_min_width(75)
		column.set_resizable(True)
		hist.append_column(column)
		column=gtk.TreeViewColumn(_("Filename"),renderer,text=2)
		column.set_min_width(75)
		column.set_resizable(True)
		hist.append_column(column)
		hist.set_model(list)
		self.history_show()
		
				
	def history_add(self,url,filename):
		md = hashlib.md5()
		md.update(url)
		md.update(filename)
		hash = md.hexdigest()
		self.history.append((hash,time.time(),url,filename))
		
	def history_clear(self):
		self.history = []
		self.history_show()
		self.history_write()
		
	def history_write(self):
		res = ""
		for (hash,timestamp,url,filename) in self.history:
			res = res + "%s\t%s\t%s\t%s\n" % (hash,timestamp,url,filename)
		f = open(self.working_dir + "/history","w")
		f.write(res)
		f.close()
		
	def history_show(self):
		hist = self.wTree.get_widget("history_list")
		list = gtk.TreeStore(gtk.gdk.Pixbuf,str,str)
		
		today = self.history_get_n_days_ago()
		if today != []:
			iter_today = list.append(None)
			list.set(iter_today,0,icons.pixbuf_from_iconname("folder"),1,_("Today"),2,"")
			for (hash,timestamp,url,filename) in today:
				list.set(list.append(iter_today),0,icons.pixbuf_from_iconname("unknown"),1,url,2,filename)
			
		yesterday = self.history_get_n_days_ago(1)
		if yesterday != []:
			iter_yesterday = list.append(None)
			list.set(iter_yesterday,0,icons.pixbuf_from_iconname("folder"),1,_("Yesterday"),2,"")
			for (hash,timestamp,url,filename) in yesterday:
				list.set(list.append(iter_yesterday),0,icons.pixbuf_from_iconname("unknown"),1,url,2,filename)
			
		two_day_ago = self.history_get_n_days_ago(2)
		if two_day_ago != []:
			iter_two_day_ago = list.append(None)
			list.set(iter_two_day_ago,0,icons.pixbuf_from_iconname("folder"),1,_("Two days ago"),2,"")
			for (hash,timestamp,url,filename) in two_day_ago:
				list.set(list.append(iter_two_day_ago),0,icons.pixbuf_from_iconname("unknown"),1,url,2,filename)
				
		earlier = self.history_get_n_days_ago(2)
		if earlier != []:
			iter_earlier = list.append(None)
			list.set(iter_earlier,0,icons.pixbuf_from_iconname("folder"),1,_("Earlier"),2,"")
			for (hash,timestamp,url,filename) in two_day_ago:
				list.set(list.append(iter_earlier),0,icons.pixbuf_from_iconname("unknown"),1,url,2,filename)
			
		hist.set_model(list)
		
	def history_get_n_days_ago(self,n=-1):
		res = []
		dt = datetime.datetime.now()
		this_day = dt.replace(hour=0,minute=0,second=0)
		for x in self.history:
			entry_time = datetime.datetime.fromtimestamp(float(x[1]))
			delta = this_day - entry_time
			if delta.days == n:
				res.append(x)
		return res
		
	def history_get_before_n_days(self,n):
		res = []
		dt = datetime.datetime.now()
		this_day = dt.replace(hour=0,minute=0,second=0)
		for x in self.history:
			entry_time = datetime.datetime.fromtimestamp(float(x[1]))
			delta = this_day - entry_time
			if delta.days > n:
				res.append(x)
		return res
		
		
	def __init_lang(self):
		"""translation stuff"""
		__builtin__._ = gettext.gettext
		locale.setlocale(locale.LC_ALL, '')
		for module in gtk.glade, gettext:
			module.bindtextdomain(self.app_name,'lang')
			module.textdomain(self.app_name)
		
	def __create_dl_list(self):
		"""init the download treeview"""
		self.dl_list = self.wTree.get_widget("dl_list")
		self.dl_list_model = gtk.TreeStore(gtk.gdk.Pixbuf,gobject.TYPE_STRING,gobject.TYPE_STRING,int,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)
		self.dl_list.set_model(self.dl_list_model)
		renderer=gtk.CellRendererPixbuf()
		column=gtk.TreeViewColumn("",renderer, pixbuf=0)
		column.set_min_width(16)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		renderer=gtk.CellRendererText()
		column=gtk.TreeViewColumn(_("Title"),renderer, text=1)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		column=gtk.TreeViewColumn(_("Status"),renderer, text=2)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		renderer=gtk.CellRendererProgress()
		column=gtk.TreeViewColumn(_("Progress"),renderer,value=3)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		renderer=gtk.CellRendererText()
		column=gtk.TreeViewColumn(_("Downloaded size"),renderer, text=4)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		column=gtk.TreeViewColumn(_("Size"),renderer, text=5)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)
		column=gtk.TreeViewColumn(_("Speed"),renderer, text=6)
		column.set_min_width(100)
		column.set_resizable(True)
		self.dl_list.append_column(column)

		self.dl_list.set_enable_tree_lines(True)
		self.dl_list_model.set(self.dl_list_model.append(None),0,icons.pixbuf_from_iconname("browser"),1,_("Downloads"),2,"",3,0,4,"",5,"",6,"")
		self.dl_list.set_model(self.dl_list_model)
		self.dl_list.expand_all()
		
	def __create_dl_menu(self):
		#create menu for the new download button
		new_dl_menu = gtk.Menu()

		new_single_dl_menuitem = misc.image_menu_item_from_name("unknown",_("Simple Download"))
		new_single_dl_menuitem.connect("activate",self.__callback_new_dl_single)
		new_single_dl_menuitem.show()
		new_dl_menu.append(new_single_dl_menuitem)

		new_gallery_dl_menuitem = misc.image_menu_item_from_name("image-x-generic",_("Gallery Download"))
		new_gallery_dl_menuitem.connect("activate",self.__callback_new_dl_gallery)
		new_gallery_dl_menuitem.show()
		new_dl_menu.append(new_gallery_dl_menuitem)
		
		new_youtube_dl_menuitem = misc.image_menu_item_from_name("video-x-generic",_("YouTube Video Download"))
		new_youtube_dl_menuitem.connect("activate",self.__callback_new_dl_youtube)
		new_youtube_dl_menuitem.show()
		new_dl_menu.append(new_youtube_dl_menuitem)
		
		new_distro_dl_menuitem = misc.image_menu_item_from_file("img/tux.png",_("Linux Distro Download"))
		new_distro_dl_menuitem.connect("activate",self.__callback_new_dl_distro)
		new_distro_dl_menuitem.show()
		new_dl_menu.append(new_distro_dl_menuitem)
		
		new_dl_menu.show()
		self.wTree.get_widget("button_new_dl").set_menu(new_dl_menu)
		
		"""create menu for the new download menu"""
		new_dl_menu = gtk.Menu()

		new_single_dl_menuitem = misc.image_menu_item_from_name("unknown",_("Simple Download"))
		new_single_dl_menuitem.connect("activate",self.__callback_new_dl_single)
		new_single_dl_menuitem.show()
		new_dl_menu.append(new_single_dl_menuitem)

		new_gallery_dl_menuitem = misc.image_menu_item_from_name("image-x-generic",_("Gallery Download"))
		new_gallery_dl_menuitem.connect("activate",self.__callback_new_dl_gallery)
		new_gallery_dl_menuitem.show()
		new_dl_menu.append(new_gallery_dl_menuitem)
		
		new_youtube_dl_menuitem = misc.image_menu_item_from_name("video-x-generic",_("YouTube Video Download"))
		new_youtube_dl_menuitem.connect("activate",self.__callback_new_dl_youtube)
		new_youtube_dl_menuitem.show()
		new_dl_menu.append(new_youtube_dl_menuitem)
		
		new_distro_dl_menuitem = misc.image_menu_item_from_file("img/tux.png",_("Linux Distro Download"))
		new_distro_dl_menuitem.connect("activate",self.__callback_new_dl_distro)
		new_distro_dl_menuitem.show()
		new_dl_menu.append(new_distro_dl_menuitem)
		
		new_dl_menu.show()
	
		self.wTree.get_widget("new_download1").set_submenu(new_dl_menu)
		
	def __create_tray(self):
		self.tray_icon = gtk.status_icon_new_from_file(self.icon)
		self.tray_icon.set_tooltip(self.app_name)
		self.tray_menu = gtk.Menu()
		
		self.tray_menu_visible = gtk.CheckMenuItem(label=_("Show %s") % self.app_name)
		self.tray_menu_visible.set_active(True)
		self.tray_menu_visible.connect("toggled",self.__callback_visible_toggled)
		
		sepa = gtk.SeparatorMenuItem()
		sepa.show()
		
		self.tray_menu_pause = misc.image_menu_item_from_stock(gtk.STOCK_MEDIA_PAUSE,_("Pause all downloads"))
		self.tray_menu_pause.connect("activate",self.__callback_pause_all)
		
		self.tray_menu_resume = misc.image_menu_item_from_stock(gtk.STOCK_MEDIA_PLAY,_("Resume all downloads"))
		self.tray_menu_resume.connect("activate",self.__callback_resume_all)
		
		self.tray_menu_quit = misc.image_menu_item_from_stock(gtk.STOCK_QUIT,_("Quit"))
		self.tray_menu_quit.connect("activate",self.__callback_quit)
		
		self.tray_icon.connect("activate",self.__callback_tray_click)
		self.tray_icon.connect("popup-menu",self.__callback_tray_rclick)
		sepb = gtk.SeparatorMenuItem()
		sepb.show()
		self.tray_menu.show()
		self.tray_menu_visible.show()
		self.tray_menu_quit.show()
		self.tray_menu_pause.show()
		self.tray_menu_resume.show()
		self.tray_menu.append(self.tray_menu_visible)
		self.tray_menu.append(sepa)
		self.tray_menu.append(self.tray_menu_pause)
		self.tray_menu.append(self.tray_menu_resume)
		self.tray_menu.append(sepb)
		self.tray_menu.append(self.tray_menu_quit)
		
	def __update_tray_tooltip(self):
		tt = self.app_name + "\n"
		tt = tt + "-----------------------------------------------------\n"
		tt = tt + _("Progress: %s %%\n") % self.global_settings.get_value("stat_dl_progress")
		tt = tt + _("Speed: %s KB/sec\n") % misc.round2(self.global_settings.get_value("stat_dl_speed"),2)
		tt = tt + _("Active downloads: %s\n") % self.active_dl_count
		tt = tt + _("Data downloaded: %s (%s)") % (misc.format_size(self.downloaded_bytes),misc.format_size(self.downloaded_bytes_session))
		gtk.gdk.threads_enter()
		self.tray_icon.set_tooltip(tt)
		gtk.gdk.threads_leave()
		
	def meta_files_add(self,list):
		for x in list:
			path = x["path"]
			url = x["url"]
			size = x["size"]
			download_date = x["date"]
			self.meta.writeString(path,"url",url)
			self.meta.writeInt(path,"size",size)
			self.meta.writeInt(path,"date",download_date)
			
	def meta_get(self,filename):
		res = {}
		res["url"] = self.meta.readString(filename,"url","")
		res["date"] = self.meta.readInt(filename,"date","")
		res["size"] = self.meta.readInt(filename,"size","")
		return res

	def load_saved_jobs(self):
		self.updater.pause()
		jl = self.saved_jobs.getSections()
		if len(jl) == 0:
			self.saved_downloads.setData({})
			self.saved_downloads.save()
		c = self.saved_jobs
		for x in jl:
			if not (self.clean_on_start_up and c.readBool(x,"done")):
				self.add_job(c.readString(x,"input",""), c.readString(x,"input_params","").split("&,&"), c.readString(x,"output",""), c.readString(x,"output_params","").split("&,&"), float(x),False)
			else:
				for y in self.saved_downloads.getSections():
					if str(self.saved_downloads.readFloat(y,"job_id")) == x:
						self.saved_downloads.removeSection(y)
				c.removeSection(x)
		self.updater.resume()
		
	def get_data_amounts(self):
		return (misc.format_size(self.downloaded_bytes),misc.format_size(self.downloaded_bytes_session))

	def __check_first_start(self):
		if not os.path.exists(os.path.expanduser("~/.%s" % self.app_short_name)):
			os.mkdir(os.path.expanduser("~/.%s" % self.app_short_name))
			
	def __check_dl_dirs(self):
		if not os.path.exists(self.download_dir):
			os.mkdir(self.download_dir)
		if not os.path.exists(self.temp_dir):
			os.mkdir(self.temp_dir)
		if not os.path.exists(self.gallery_dir):
			os.mkdir(self.gallery_dir)
		if not os.path.exists(self.video_dir):
			os.mkdir(self.video_dir)
		if not os.path.exists(self.audio_dir):
			os.mkdir(self.audio_dir)

	#download functions
	def add_job(self,input,input_params,output,output_params,id=None,get_downloads=True):
		if id == None:
			id = time.time()
		job = download.Job(id,self,input,input_params,output,output_params,get_downloads,self.global_settings)
		self.job_list.append(job)
		self.id_tree.add_child(misc.Id_tree(id))
		self.global_settings.get_value("logger_main").info("Added new job. Id: %s" % id)

		input_params_str = ""
		for i in range(0,len(input_params)):
			input_params_str += str(input_params[i])
			if i < len(input_params) - 1:
				input_params_str += "&,&"
		output_params_str = ""
		for i in range(0,len(output_params)):
			output_params_str += output_params[i]
			if i < len(output_params) - 1:
				output_params_str += "&,&"
		self.saved_jobs.writeString(str(id),"input",input)
		self.saved_jobs.writeString(str(id),"input_params",input_params_str)
		self.saved_jobs.writeString(str(id),"output",output)
		self.saved_jobs.writeString(str(id),"output_params",output_params_str)
		self.saved_jobs.writeBool(str(id),"done",False)

	def add_single_download(self,download):
		iter = self.dl_list_model.get_iter(self.get_job_path_by_id(download.get_job_id()))
		new_iter = self.dl_list_model.append(iter)
		self.id_tree.get_child_by_id(download.get_job_id()).add_child(misc.Id_tree(download.get_id()))
		self.get_job(download.get_job_id()).add_dl(download)
		if self.saved_downloads.readFloat(download.get_url(),"percentage",-1.0) == -1.0:
			self.saved_downloads.writeFloat(download.get_url(),"percentage",0.0)
			self.saved_downloads.writeBool(download.get_url(),"done",False)
			self.saved_downloads.writeInt(download.get_url(),"local_size",0)
			self.saved_downloads.writeInt(download.get_url(),"remote_size",0)
			self.saved_downloads.writeString(download.get_url(),"title",download.get_title())
			self.saved_downloads.writeFloat(download.get_url(),"job_id",download.get_job_id())
			self.dl_list_model.set(new_iter,0,icons.get_file_icon(download.get_url()),1,download.get_title(),2," - ",3,0,4," - ",5," - ",6,"-")
		else:
			p = self.saved_downloads.readFloat(download.get_url(),"percentage",0.0)
			ls = self.saved_downloads.readInt(download.get_url(),"local_size",0)
			rs = self.saved_downloads.readInt(download.get_url(),"remote_size",0)
			self.dl_list_model.set(new_iter,0,icons.get_file_icon(download.get_url()),1,download.get_title(),2," - ",3,int(p*100),4,misc.format_size(ls),5,misc.format_size(rs),6,"-")

		download.set_status(0)
		self.downloads.append(download)
		
	def cancel_all_downloads(self):
		l = self.id_tree.get_children()
		while l != {}:
			self.cancel_job((0,0))
			l = self.id_tree.get_children()

	def cancel_job(self,path):
		id = self.id_tree.get_id_by_path(path)
		for job in self.job_list:
			if job.get_id() == id:
				break
		n = job.get_download_count()
		la = self.id_tree.get_children()
		l = la[path[1]].get_children()
		while l != {}:
			self.cancel_download((path[0],path[1],0),True,n)
			l = la[path[1]].get_children()
		for i in range(0,len(self.job_list)):
			if self.job_list[i].get_id() == id:
				del self.job_list[i]
				break
				
		self.saved_jobs.removeSection(id)
		self.saved_jobs.removeSection(str(id))
		self.id_tree.remove_child(id)
		self.global_settings.get_value("logger_main").info("Job deleted.")

	def cancel_download(self,path,j=False,n=-1):
		id = self.id_tree.get_id_by_path(path)
		for download in self.downloads:
			if download.get_id() == id:
				break
		if download != None:
			url = download.get_url()
			parent_path = (path[0],path[1])
			if n == 1 and j == False:
				self.cancel_job(parent_path)
			else:
				download.pause()
				self.saved_downloads.removeSection(url)
				if os.path.exists(download.get_temp_filename()):
					os.remove(download.get_temp_filename())
				for i in range(0,len(self.downloads)):
					if self.downloads[i].get_id() == id:
						del self.downloads[i]
						break
				download.delete_status()
				self.id_tree.remove_child(id)
				self.global_settings.get_value("logger_main").info("Download '%s' deleted." % url)

	#download storage managment functions
	def get_job(self,id):
		res = None
		for x in self.job_list:
			if str(x.get_id()) == str(id):
				res = x
				break
		return res

	def get_download_by_url(self,url):
		for x in self.downloads:
			if x.get_url() == url:
				return x
		return None
		
	def get_download_by_id(self,id):
		for x in self.downloads:
			if x.get_id() == id:
				return x
		return None
		
	def get_job_path_by_id(self,id):
		l = self.id_tree.get_children()
		for (i,job) in l.iteritems():
			if job.get_id() == str(id):
				return (0,i)
		return None

	def get_dl_list(self):
		return self.dl_list

	def get_dl_list_model(self):
		return self.dl_list_model
		
	def update_status_bar(self):
		bar = self.wTree.get_widget("statusbar")
		id = self.global_settings.get_value("statusbar_id")
		txt = _("Downloads: %s\tActive downloads: %s\tSpeed: %s KB/s") % (str(self.global_settings.get_value("stat_dl_count")),str(self.global_settings.get_value("stat_dl_active_count")),misc.round2(self.global_settings.get_value("stat_dl_speed"),2))
		bar.push(id,txt)
		
	#misc functions
	def update_status(self):
		self.active_dl_count = 0
		for x in self.downloads:
			if x.is_active():
				self.active_dl_count = self.active_dl_count + 1
		self.global_settings.get_value("logger_threading").debug("%s active threads." % threading.activeCount())
		active_jobs = {}
		root_path = (0)
		root_iter = self.dl_list_model.get_iter(root_path)
		root_percentage = 0
		root_speed = 0.0
		job_list = self.id_tree.get_children()
		i = -1
		for (i,job_tree) in job_list.iteritems():
			download_list = job_tree.get_children()
			job_path = (0,i)
			job_iter = self.dl_list_model.get_iter(job_path)
			job = self.get_job(job_tree.get_id())
			active_jobs[str(job.get_id())] = False
			job_percentage = 0.0
			job_speed = 0.0
			job_status = job.get_message()
			job_title = job.get_title()
			j = -1
			for (j,download_tree) in download_list.iteritems():
				try:
					download_path = (0,i,j)
					download_iter = self.dl_list_model.get_iter(download_path)
					download = self.get_download_by_id(download_tree.get_id())
					if not download.is_active() and self.active_dl_count < self.max_dl_num and threading.activeCount() < self.max_thread_num and not download.is_stopped():
						download.download()
						self.active_dl_count = self.active_dl_count + 1
					if download.is_active():
						active_jobs[str(download.get_job_id())] = True
					bytes = download.get_downladed_bytes()
					self.downloaded_bytes = self.downloaded_bytes + bytes
					self.downloaded_bytes_session = self.downloaded_bytes_session + bytes
					download_percentage = int(download.get_percentage()*100)
					download_speed = misc.round2(download.get_speed()/1024,2)
					if download.get_status() == 0:
						download_speed = 0.0
					if download.get_message() == "":
						download_status = self.status_str[download.get_status()]
					else:
						download_status = download.get_message()
						job_status = _("One or more errors occured")
					download_title = download.get_title()
					download_remote_size,download_local_size = download.get_size()
					job_speed = job_speed + download_speed
					job_percentage = job_percentage + download_percentage
					self.dl_list_model.set(download_iter,1,download_title,2,download_status,3,download_percentage,4,misc.format_size(download_local_size),5,misc.format_size(download_remote_size),6,"%s KB/s" % download_speed)
				except:
					self.global_settings.get_value("logger_main").debug("Download not found.")
			if j != -1:
				job_percentage = int(job_percentage/(j+1))
			job_speed = misc.round2(job_speed,2)
			if job_status == "" and (str(job.get_id()) in active_jobs) and active_jobs[str(job.get_id())]:
				job_status = _("Downloading")
			self.dl_list_model.set(job_iter,1,job_title,2,job_status,3,job_percentage,6,"%s KB/s" % job_speed)
			root_percentage = root_percentage + job_percentage
			root_speed = root_speed + job_speed

		if i != -1:
			root_percentage = int(root_percentage/(i+1))
		root_speed = misc.round2(root_speed,2)
		self.dl_list_model.set(root_iter,3,root_percentage,6,"%s KB/s" % root_speed)
		
		self.config.writeInt("stats","downloaded_bytes",self.downloaded_bytes)
		self.global_settings.set_value("stat_downloaded_bytes",self.downloaded_bytes)
		self.global_settings.set_value("stat_downloaded_bytes_session",self.downloaded_bytes_session)
		self.global_settings.set_value("stat_dl_speed",root_speed)
		self.global_settings.set_value("stat_dl_count",len(self.downloads))
		self.global_settings.set_value("stat_dl_active_count",self.active_dl_count)
		self.global_settings.set_value("stat_dl_progress",root_percentage)
		self.global_settings.get_value("logger_main").info("Statistics: %s (%s) downloaded" % (misc.format_size(self.downloaded_bytes),misc.format_size(self.downloaded_bytes_session)))

		self.__update_tray_tooltip()
		self.update_status_bar()
		self.__callback_dl_list_click(None,None)

	def save_settings(self,settings):
		self.config.writeString("app","download_dir",settings["app_dl_dir"])
		self.config.writeString("app","temp_dir",settings["app_temp_dir"])
		self.config.writeInt("app","max_dl_count",settings["app_max_dl_n"])
		self.config.writeBool("app","clean_on_startup",settings["app_cleanup"])
		self.config.writeString("input-gallery","gallery_dir",settings["gallery_dl_dir"])
		self.config.writeString("input-video","video_dir",settings["video_dl_dir"])
		
		self.download_dir = self.config.readString("app","download_dir",os.path.expanduser("~/Downloads"))
		self.temp_dir = self.config.readString("app","temp_dir",os.path.expanduser("~/Downloads/.temp"))
		self.gallery_dir = self.config.readString("input-gallery","gallery_dir",os.path.expanduser("~/Downloads/Galleries"))
		self.video_dir = self.config.readString("input-video","video_dir",os.path.expanduser("~/Downloads/Videos"))
		self.max_dl_num = self.config.readInt("app","max_dl_count",4)
		self.clean_on_start_up = self.config.readBool("app","clean_on_startup",False)
		self.max_thread_num = self.config.readInt("app","max_thread_count",10)
		self.global_settings.get_value("logger_main").info(_("Settings saved."))
		
	def update_sidebar(self,path,iter,model):
		if self.wTree.get_widget("sidebar").get_position() > 0:
			self.wTree.get_widget("viewmenu_sidebar").set_active(True)
		else:
			self.wTree.get_widget("viewmenu_sidebar").set_active(False)
		if path == None and iter == None and model == None:
			self.wTree.get_widget("info_title").set_text(_("<b>Title</b>"))
			self.wTree.get_widget("info_title").set_use_markup(True)
			self.wTree.get_widget("info_status").set_text("-")
			self.wTree.get_widget("info_progress").set_text("-")
			self.wTree.get_widget("info_url").set_text("")
			self.wTree.get_widget("info_size").set_text("-")
			self.wTree.get_widget("info_speed").set_text("-")
		else:
			dl = None
			title = model.get_value(iter,1)
			status = model.get_value(iter,2)
			progress = model.get_value(iter,3)
			url = ""
			remote_size = model.get_value(iter,5)
			local_size = model.get_value(iter,4)
			speed = model.get_value(iter,6)
			size = "%s (%s)" % (remote_size,local_size)
			if len(path) == 3:
				id = self.id_tree.get_id_by_path(path)
				dl = self.get_download_by_id(id)
				if dl != None:
					url = dl.get_url()
			if status == "":
				status = "-"
			if remote_size == "" or local_size == "":
				size = "-"
			
			self.wTree.get_widget("info_title").set_text("<b>%s</b>" % title)
			self.wTree.get_widget("info_title").set_use_markup(True)
			self.wTree.get_widget("info_status").set_text(status)
			self.wTree.get_widget("info_progress").set_text("%s %%" % progress)
			self.wTree.get_widget("info_url").set_text(url)
			self.wTree.get_widget("info_size").set_text(size)
			self.wTree.get_widget("info_speed").set_text(speed)

	#callbacks
	def __callback_quit(self,widget):
		self.config.writeBool("app","sidebar-show",(self.wTree.get_widget("sidebar").get_position() > 200))
		self.config.writeInt("app","sidebar-position",self.wTree.get_widget("sidebar").get_position())
		self.history_write()
		self.updater.pause()
		self.updater.join()
		gtk.main_quit()
		for x in self.downloads:
			x.pause()
		self.global_settings.get_value("logger_main").info(_("%s closed.") % self.app_name)
		self.global_settings.get_value("logger_main").info(_("Bye."))
		
	def __callback_tray_click(self,icon):
		self.__visible_toggled = False
		if self.showing:
			self.wTree.get_widget("window_main").hide()
			self.showing = False
			self.tray_menu_visible.set_active(False)
		else:
			self.wTree.get_widget("window_main").show_all()
			self.showing = True
			self.tray_menu_visible.set_active(True)

	def __callback_tray_rclick(self,icon,button,t):
		self.tray_menu.popup(None, None, gtk.status_icon_position_menu,button, t, self.tray_icon)
		
	def __callback_visible_toggled(self,widget):
		if self.__visible_toggled:
			if self.showing:
				self.wTree.get_widget("window_main").hide()
				self.showing = False
			else:
				self.wTree.get_widget("window_main").show_all()
				self.showing = True
		self.__visible_toggled = True
		
	def __callback_pause_all(self,widget):
		for x in self.downloads:
			x.pause()
		
	def __callback_resume_all(self,widget):
		for x in self.downloads:
			x.download()

	def __callback_new_dl_single(self,widget):
		single_window = input_single.Input_single_window(self,self.global_settings)

	def __callback_new_dl_gallery(self,widget):
		gallery_window = input_gallery.Input_gallery_window(self,self.global_settings,self.config.readInt("input-gallery","fn_zerofill",2))
		
	def __callback_new_dl_youtube(self,widget):
		youtube_window = input_youtube.Input_youtube_window(self,self.global_settings)
		
	def __callback_new_dl_distro(self,widget):
		distro_window = input_distro.Input_distro_window(self,self.global_settings)
		
	def __callback_button_history_clear_clicked(self,widget):
		self.history_clear()
		
	def __callback_history_list_clicked(self,widget,event):
		if event.type == gtk.gdk._2BUTTON_PRESS:
			selection = widget.get_selection()
			if selection.count_selected_rows() > 0:
				(model,iter) = selection.get_selected()
				path = model.get_path(iter)
				if len(path) == 2:
					url = model.get_value(iter,1)
					self.add_job("input_single",[url],"output_single",[])

	def __callback_dl_list_click(self,widget,event):
		if event == None or event.type == gtk.gdk.BUTTON_PRESS:
			selection = self.dl_list.get_selection()
			if selection.count_selected_rows() > 0:
				self.wTree.get_widget("button_pause_dl").set_sensitive(True)
				self.wTree.get_widget("editmenu_pause_dl").set_sensitive(True)
				self.wTree.get_widget("button_start_dl").set_sensitive(True)
				self.wTree.get_widget("editmenu_start_dl").set_sensitive(True)
				self.wTree.get_widget("button_del_dl").set_sensitive(True)
				self.wTree.get_widget("editmenu_remove_dl").set_sensitive(True)
				(model,iter) = selection.get_selected()
				path = model.get_path(iter)
				self.update_sidebar(path,iter,model)
				if iter != None and len(path) == 3:
					id = self.id_tree.get_id_by_path(model.get_path(iter))
					dl = self.get_download_by_id(id)
					if dl.get_status() == 0 or dl.get_status() == 1:
						self.wTree.get_widget("button_pause_dl").set_sensitive(False)
						self.wTree.get_widget("editmenu_pause_dl").set_sensitive(False)
						self.wTree.get_widget("button_start_dl").set_sensitive(True)
						self.wTree.get_widget("editmenu_start_dl").set_sensitive(True)
					else:
						self.wTree.get_widget("button_pause_dl").set_sensitive(True)
						self.wTree.get_widget("editmenu_pause_dl").set_sensitive(True)
						self.wTree.get_widget("button_start_dl").set_sensitive(False)
						self.wTree.get_widget("editmenu_start_dl").set_sensitive(False)
			else:
				self.wTree.get_widget("button_pause_dl").set_sensitive(False)
				self.wTree.get_widget("editmenu_pause_dl").set_sensitive(False)
				self.wTree.get_widget("button_start_dl").set_sensitive(False)
				self.wTree.get_widget("editmenu_start_dl").set_sensitive(False)
				self.wTree.get_widget("button_del_dl").set_sensitive(False)
				self.wTree.get_widget("editmenu_remove_dl").set_sensitive(False)
				self.update_sidebar(None,None,None)

	def __callback_button_pause_clicked(self,widget):
		selection = self.dl_list.get_selection()
		if selection.count_selected_rows > 0:
			(model,iter) = selection.get_selected()
			path = model.get_path(iter)
			if len(path) == 1:
				for x in self.downloads:
					x.pause()
			elif len(path) == 2:
				job_id = self.id_tree.get_id_by_path(path)
				for x in self.downloads:
					if str(x.get_job_id()) == job_id:
						x.pause()
			elif len(path) == 3:
				download_id = self.id_tree.get_id_by_path(path)
				for x in self.downloads:
					if x.get_id() == download_id:
						x.pause()
						break
			
			self.wTree.get_widget("button_pause_dl").set_sensitive(False)
			self.wTree.get_widget("button_start_dl").set_sensitive(True)

	def __callback_button_start_clicked(self,widget):
		selection = self.dl_list.get_selection()
		if selection.count_selected_rows > 0:
			(model,iter) = selection.get_selected()
			path = model.get_path(iter)
			if len(path) == 1:
				for x in self.downloads:
					x.download()
			elif len(path) == 2:
				job_id = self.id_tree.get_id_by_path(path)
				for x in self.downloads:
					if str(x.get_job_id()) == job_id:
						x.download()
			elif len(path) == 3:
				download_id = self.id_tree.get_id_by_path(path)
				for x in self.downloads:
					if x.get_id() == download_id:
						x.download()
						break
			
			self.wTree.get_widget("button_pause_dl").set_sensitive(True)
			self.wTree.get_widget("button_start_dl").set_sensitive(False)

	def __callback_button_del_clicked(self,widget):
		self.updater.pause()
		selection = self.dl_list.get_selection()
		if selection.count_selected_rows > 0:
			(model,iter) = selection.get_selected()
			path = model.get_path(iter)
			l = len(path)
			if l == 1:
				self.cancel_all_downloads()
			elif l == 3:
				self.cancel_download(path)
			elif l == 2:
				self.cancel_job(path)
			model.remove(iter)
			if l == 1:
				self.dl_list_model.set(self.dl_list_model.append(None),0,icons.pixbuf_from_iconname("browser"),1,_("Downloads"),2,"",3,0,4,"",5,"",6,"")
		self.updater.resume()

	def __callback_button_clear_clicked(self,widget):
		self.updater.pause()
		time.sleep(1)
		jobs = self.job_list[:]
		"""for (x,y) in self.id_tree.get_children().copy().iteritems():
			job = self.get_job(y.get_id())
			if job.is_done():
				self.cancel_job((0,x))
				self.dl_list_model.remove(self.dl_list_model.get_iter((0,x)))"""
		cancel = None
		for (x,y) in self.id_tree.get_children().iteritems():
			job = self.get_job(y.get_id())
			if job.is_done():
				cancel = (job,x)
		if cancel != None:
			self.cancel_job((0,cancel[1]))
			self.dl_list_model.remove(self.dl_list_model.get_iter((0,cancel[1])))
		while cancel != None:
			cancel = None
			for (x,y) in self.id_tree.get_children().iteritems():
				job = self.get_job(y.get_id())
				if job.is_done():
					cancel = (job,x)
			if cancel != None:
				self.cancel_job((0,cancel[1]))
				self.dl_list_model.remove(self.dl_list_model.get_iter((0,cancel[1])))
		
		self.updater.resume()

	def __callback_button_settings_clicked(self,widget):
		settings = {"app_dl_dir":self.download_dir,
				"app_temp_dir":self.temp_dir,
				"app_max_dl_n":self.max_dl_num,
				"app_cleanup":self.clean_on_start_up,
				"gallery_dl_dir":self.gallery_dir[len(self.download_dir)+1:],
				"video_dl_dir":self.video_dir[len(self.download_dir)+1:]}
		settings_window = window_settings.Window_settings(self,settings)
		
	def __callback_button_log_clicked(self,widget):
		log_window = window_log.Window_log(self)
		
	def __callback_button_statistics_clicked(self,widget):
		stat = window_statistics.Window_statistics(self)
		
	def __callback_button_dl_dir_clicked(self,widget):
		os.spawnvp(os.P_NOWAIT,'nautilus',['nautilus',self.download_dir])
		
	def __callback_menu_about_clicked(self,widget):
		abd = gtk.AboutDialog()
		gtk.about_dialog_set_url_hook(self.__callback_url_clicked,None)
		gtk.about_dialog_set_email_hook(self.__callback_mail_clicked,None)
		abd.set_name(self.app_name)
		abd.set_authors(["Sven Festersen <sven@sven-festersen.de>"])
		f = file("LICENSE")
		abd.set_license(f.read())
		f.close()
		abd.set_copyright(u"\xa9 2007 by Sven Festersen")
		abd.set_logo(gtk.gdk.pixbuf_new_from_file(self.icon))
		abd.set_website(self.website)
		abd.set_website_label(self.website)
		abd.set_version(self.version)
		abd.run()
		abd.destroy()
		
	def __callback_url_clicked(self,dialog,link,data):
		os.spawnvp(os.P_NOWAIT,'htmlview',['htmlview',link])
		
	def __callback_mail_clicked(self,dialog,link,data):
		os.spawnvp(os.P_NOWAIT,'launchmail',['launchmail',link])
		
	def __callback_sidebar_toggled(self,widget):
		val = widget.get_active()
		if val:
			self.wTree.get_widget("sidebar").set_position(300)
			widget.set_active(True)
		else:
			self.wTree.get_widget("sidebar").set_position(0)
			widget.set_active(False)


class Update_thread(threading.Thread):
	def __init__(self,downloader,settings):
		threading.Thread.__init__(self)
		self.downloader = downloader
		self.__stop_event = threading.Event()
		self.__pause_event = threading.Event()
		self.settings = settings
		self.paused = False
		self.settings.get_value("logger_threading").debug("Update thread started.")

	def run(self):
		while not self.__stop_event.isSet():
			if not self.__pause_event.isSet():
				self.downloader.update_status()
				self.paused = False
			else:
				self.paused = True
			time.sleep(1)

	def join(self,timeout=None):
		self.__stop_event.set()
		self.settings.get_value("logger_threading").debug("Update thread stopped.")
		threading.Thread.join(self, timeout)

	def pause(self):
		self.__pause_event.set()

	def resume(self):
		self.__pause_event.clear()
		
	def is_paused(self):
		return self.paused

if __name__ == "__main__":
	app = Downloader()
