# simpleCSSConfig for Python 1.1
# Copyright 2007 Sven Festersen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import os

#start constants
TYPE_STRING = 'simpleCSSConfig_String'
TYPE_BOOL = 'simpleCSSConfig_Boolean'
TYPE_INT = 'simpleCSSConfig_Integer'
TYPE_FLOAT = 'simpleCSSConfig_Float'
TYPE_NONE = 'simpleCSSConfig_None'
#end constants

class ConfigFile:
	def __init__(self,filename,virtual=False):
		self.filename = filename
		self.virtual = virtual
		self.__isWriteable()
		self.__read()

	def __isWriteable(self):
		self.write = os.access(self.filename,os.W_OK)
		if self.write == False:
			self.virtual = True

	def __makeSafe(self,lines):
		temp = []
		for x in lines:
			x = x.strip()
			l = x.split('#')
			if len(l) > 1:
				x = l[0]
				x = x.strip()
			if x != '' and x[0] != '#':
				temp.append(x)
		return temp

	def __read(self):
		self.data = {}
		try:
			file = open(self.filename,'r')
		except IOError:
			print 'Config file does not exist. Creating it.'
			file = open(self.filename,'w')
			file.close()
			file = open(self.filename,'r')
		lines = file.readlines()
		lines = self.__makeSafe(lines)
		file.close()
		section = ''
		for x in lines:
			c = x.split(";")
			if len(c) == 1:
				c.append("")
			elif len(c) > 2:
				t = ''
				for i in range(0,len(c)-1):
					t += c[i]
					t += ";"
				t = t[:len(t)-1]
				c = [t,c[len(c)-1]]
			x = c[0].strip(";")
			if x.find('{') != -1 and section == '':
				for i in range(0,x.find('{')):
					section = section + x[i]
				self.data[section] = {}
			elif x.find('}') != -1 and section != '':
				section = ''
			elif x.find('{') == -1 and x.find('}') == -1 and section != '':
				temp = x.split(': ')
				temp.append('none')
				key = temp[0]
				value = temp[1]
				self.data[section][key] = [value.strip("'*"),self.__getType(value),c[1]]

	def __getType(self,value):
		value = value.strip()
		if value[0] == "'" and value[len(value)-1] == "'":
			return TYPE_STRING
		elif value[0] == "*" and value[len(value)-1] == "*":
			return TYPE_BOOL
		elif value[0] != "'" and value[len(value)-1] != "'" and value[0] != "*" and value[len(value)-1] != "*" and value.find(".") != -1:
			return TYPE_FLOAT
		elif value[0] != "'" and value[len(value)-1] != "'" and value[0] != "*" and value[len(value)-1] != "*" and value.find(".") == -1:
			return TYPE_INT
		else:
			return TYPE_NONE

	def __store(self,force=False):
		if not self.virtual or force:
			temp = ""
			dat = self.data.copy()
			for x,y in dat.iteritems():
				temp = temp + x + "{\n"
				for a,b in y.iteritems():
					delim = ''
					if b[1] == TYPE_STRING:
						delim = "'"
					elif b[1] == TYPE_BOOL:
						delim = "*"
					temp += "%s: %s%s%s;%s\n" % (a,delim,b[0],delim,b[2])
					#temp = temp + a + ": " + b[0] + "\n"
				temp = temp + "}\n\n"
			file = open(self.filename,'w')
			file.write(temp.strip())
			file.close()

	#start file operations
	def reload(self):
		self.__read()

	def save(self):
		self.__store(True)
	#end file operations

	#start write methods
	def writeString(self,section,key,value):
		if section in self.data:
			c = ''
			if key in self.data[section]:
				c = self.data[section][key][2]
			self.data[section][key] = [value,TYPE_STRING,c]
		else:
			self.data[section] = {}
			self.data[section][key] = [value,TYPE_STRING,'']
		self.__store()

	def writeInt(self,section,key,value):
		if section in self.data:
			c = ''
			if key in self.data[section]:
				c = self.data[section][key][2]
			self.data[section][key] = [str(value),TYPE_INT,c]
		else:
			self.data[section] = {}
			self.data[section][key] = [str(value),TYPE_INT,'']
		self.__store()

	def writeFloat(self,section,key,value):
		if str(value).find(".") == -1:
			value = float(value)
		if section in self.data:
			c = ''
			if key in self.data[section]:
				c = self.data[section][key][2]
			self.data[section][key] = [str(value),TYPE_FLOAT,c]
		else:
			self.data[section] = {}
			self.data[section][key] = [str(value),TYPE_FLOAT,'']
		self.__store()

	def writeBool(self,section,key,val):
		value = 'False'
		if val:
			value = 'True'
		if section in self.data:
			c = ''
			if key in self.data[section]:
				c = self.data[section][key][2]
			self.data[section][key] = [str(value),TYPE_BOOL,c]
		else:
			self.data[section] = {}
			self.data[section][key] = [str(value),TYPE_BOOL,'']
		self.__store()
	#end write methods

	#start read methods
	def readInt(self,section,key,default=-1):
		if (not section in self.data) or (not key in self.data[section]) or not self.data[section][key][1] == TYPE_INT:
			return default
		else:
			return int(self.data[section][key][0])

	def readFloat(self,section,key,default=-1.0):
		if (not section in self.data) or (not key in self.data[section]) or not self.data[section][key][1] == TYPE_FLOAT:
			return default
		else:
			return float(self.data[section][key][0])

	def readString(self,section,key,default='none'):
		if (not section in self.data) or (not key in self.data[section]) or not self.data[section][key][1] == TYPE_STRING:
			return default
		else:
			return str(self.data[section][key][0])

	def readBool(self,section,key,default=True):
		if (not section in self.data) or (not key in self.data[section]) or not self.data[section][key][1] == TYPE_BOOL:
			return default
		else:
			if self.data[section][key][0] == 'True':
				return True
			else:
				return False
	#end read methods

	#start information methods
	def getSections(self):
		return self.data.keys()

	def getKeysInSection(self,section):
		if not section in self.data:
			res = []
		else:
			res = self.data[section].keys()
		return res

	def removeSection(self,section):
		if section in self.data:
			del self.data[section]
			self.__store()

	def getFileSize(self):
		return os.stat(self.filename).st_size

	def isWriteable(self):
		return self.write
	#end information methods

	#start multiple instances methods
	def join(self,c):
		ds = self.getData()
		dc = c.getData()
		for x,y in dc.iteritems():
			if not x in ds:
				ds[x] = y
			else:
				for a,b in y.iteritems():
					if not a in ds[x]:
						ds[x][a] = b
		temp = ConfigFile(self.filename)
		temp.setData(ds)
		return temp

	def meet(self,c):
		ds = self.getData()
		dc = c.getData()
		dn = {}
		for x,y in dc.iteritems():
			if x in ds:
				for a,b in y.iteritems():
					if a in ds[x] and ds[x][a] == b:
						if not x in dn:
							dn[x] = {}
						dn[x][a] = b
		temp = ConfigFile(self.filename)
		temp.setData(dn)
		return temp

	def complement(self,c):
		ds = self.getData()
		dc = c.getData()
		dn = {}
		for x,y in ds.iteritems():
			if x in dc:
				for a,b in y.iteritems():
					if not a in dc[x] or not dc[x][a] == b:
						if not x in dn:
							dn[x] = {}
						dn[x][a] = b
			else:
				dn[x] = y
		temp = ConfigFile(self.filename)
		temp.setData(dn)
		return temp

	#end multiple instances methods

	#start misc methods
	def setData(self,data):
		self.data = data

	def getData(self):
		return self.data

	def setFilename(self,filename):
		self.filename = filename

	def getFilename(self):
		return self.filename
	#end misc methods

	#start infix operations
	def __add__(self,c):
		return self.join(c)

	def __sub__(self,c):
		return self.complement(c)
	#end infix operations

if __name__ == '__main__':
	print
	print " simpleCSSConfig for Python version 1.1, Copyright (C) 2007 Sven Festersen"
	print
	print " simpleCSSConfig for Python comes with ABSOLUTELY NO WARRANTY."
	print " This is free software, and you are welcome to redistribute it under"
	print " certain conditions; See LICENSE file that came along with this software"
	print " for details."
	print
